"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeSubquery = void 0;
function decodeSubquery(query, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const cacheKey = JSON.stringify(query._conditions);
        query.options.decodeSubqueryPromises = query.options.decodeSubqueryPromises || {};
        if (query.options.decodeSubqueryPromises[cacheKey]) {
            console.warn("This query is already decoding, please wait for the returned Promise.");
            return query.options.decodeSubqueryPromises[cacheKey].then((obj) => (query._conditions = obj));
        }
        const decodeRecursive = function (obj, referenceFields, parentKey) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!referenceFields) {
                    referenceFields = {};
                    const paths = query.model.schema.paths;
                    Object.keys(paths).forEach((fieldKey) => {
                        const field = paths[fieldKey];
                        if (field.constructor.name === "ObjectId" && field.options.ref) {
                            referenceFields[fieldKey] = field.options.ref;
                        }
                        else if (field.constructor.name === "SchemaArray" && field.$embeddedSchemaType.constructor.name === "ObjectId" && field.$embeddedSchemaType.options.ref) {
                            referenceFields[fieldKey] = field.$embeddedSchemaType.options.ref;
                        }
                    });
                }
                yield Promise.all(Object.keys(obj).map((key) => __awaiter(this, void 0, void 0, function* () {
                    const value = obj[key];
                    if (value && typeof value === "object" && value.$subquery) {
                        const modelName = referenceFields[parentKey || key];
                        if (modelName) {
                            if (options.initQuery) {
                                yield options.initQuery(query, parentKey || key, obj, modelName);
                            }
                            const referenceModel = query.model.db.model(modelName);
                            let subquery = referenceModel.where(value.$subquery).select("_id");
                            if (options.beforeQuery) {
                                yield options.beforeQuery(subquery, query);
                            }
                            let operator = value.$operator;
                            let _options = value.$options || {};
                            delete value.$subquery;
                            delete value.$operator;
                            delete value.$options;
                            subquery.setOptions(_options);
                            let res;
                            if (options.resolve) {
                                res = yield options.resolve(subquery, query);
                            }
                            else {
                                res = yield subquery.find();
                            }
                            res = Array.isArray(res) ? res.map((r) => r._id) : res && typeof res === "object" && "_id" in res ? res._id : new String(res);
                            if (/^\$/.test(key) && !operator) {
                                obj[key] = res;
                            }
                            else {
                                value[operator || "$in"] = res;
                            }
                        }
                    }
                    else if (value && typeof value === "object") {
                        yield decodeRecursive(value, referenceFields, !/^\$/.test(key) && !Array.isArray(obj) ? key : parentKey);
                    }
                })));
                return obj;
            });
        };
        query.options.decodeSubqueryPromises[cacheKey] = decodeRecursive(JSON.parse(JSON.stringify(query._conditions)));
        return query.options.decodeSubqueryPromises[cacheKey].then((obj) => (query._conditions = obj));
    });
}
exports.decodeSubquery = decodeSubquery;
//# sourceMappingURL=utils.js.map